pub struct DVec4(/* private fields */);Expand description
A 4-dimensional vector.
Implementations
Sourceimpl DVec4
impl DVec4
Sourcepub fn truncate(self) -> DVec3
pub fn truncate(self) -> DVec3
Creates a Vec3 from the x, y and z elements of self, discarding w.
Truncation to Vec3 may also be performed by using self.xyz() or Vec3::from().
To truncate to Vec3A use Vec3A::from().
Sourcepub fn select(mask: BVec4, if_true: DVec4, if_false: DVec4) -> DVec4
pub fn select(mask: BVec4, if_true: DVec4, if_false: DVec4) -> DVec4
Creates a vector from the elements in if_true and if_false, selecting which to use
for each element of self.
A true element in the mask uses the corresponding element from if_true, and false
uses the element from if_false.
Sourcepub fn min(self, other: DVec4) -> DVec4
pub fn min(self, other: DVec4) -> DVec4
Returns a vector containing the mininum values for each element of self and other.
In other words this computes [self.x.max(other.x), self.y.max(other.y), ..].
Sourcepub fn max(self, other: DVec4) -> DVec4
pub fn max(self, other: DVec4) -> DVec4
Returns a vector containing the maximum values for each element of self and other.
In other words this computes [self.x.max(other.x), self.y.max(other.y), ..].
Sourcepub fn clamp(self, min: DVec4, max: DVec4) -> DVec4
pub fn clamp(self, min: DVec4, max: DVec4) -> DVec4
Component-wise clamping of values, similar to f32::clamp.
Each element in min must be less-or-equal to the corresponing element in max.
Panics
Will panic if min is greater than max when glam_assert is enabled.
Sourcepub fn min_element(self) -> f64
pub fn min_element(self) -> f64
Returns the horizontal minimum of self.
In other words this computes min(x, y, ..).
Sourcepub fn max_element(self) -> f64
pub fn max_element(self) -> f64
Returns the horizontal maximum of self.
In other words this computes max(x, y, ..).
Sourcepub fn cmpeq(self, other: DVec4) -> BVec4
pub fn cmpeq(self, other: DVec4) -> BVec4
Returns a vector mask containing the result of a == comparison for each element of
self and other.
In other words, this computes [self.x == other.x, self.y == other.y, ..] for all
elements.
Sourcepub fn cmpne(self, other: DVec4) -> BVec4
pub fn cmpne(self, other: DVec4) -> BVec4
Returns a vector mask containing the result of a != comparison for each element of
self and other.
In other words this computes [self.x != other.x, self.y != other.y, ..] for all
elements.
Sourcepub fn cmpge(self, other: DVec4) -> BVec4
pub fn cmpge(self, other: DVec4) -> BVec4
Returns a vector mask containing the result of a >= comparison for each element of
self and other.
In other words this computes [self.x >= other.x, self.y >= other.y, ..] for all
elements.
Sourcepub fn cmpgt(self, other: DVec4) -> BVec4
pub fn cmpgt(self, other: DVec4) -> BVec4
Returns a vector mask containing the result of a > comparison for each element of
self and other.
In other words this computes [self.x > other.x, self.y > other.y, ..] for all
elements.
Sourcepub fn cmple(self, other: DVec4) -> BVec4
pub fn cmple(self, other: DVec4) -> BVec4
Returns a vector mask containing the result of a <= comparison for each element of
self and other.
In other words this computes [self.x <= other.x, self.y <= other.y, ..] for all
elements.
Sourcepub fn cmplt(self, other: DVec4) -> BVec4
pub fn cmplt(self, other: DVec4) -> BVec4
Returns a vector mask containing the result of a < comparison for each element of
self and other.
In other words this computes [self.x < other.x, self.y < other.y, ..] for all
elements.
Sourcepub fn from_slice(slice: &[f64]) -> DVec4
pub fn from_slice(slice: &[f64]) -> DVec4
Creates a vector from the first N values in slice.
Panics
Panics if slice is less than N elements long.
Sourcepub fn write_to_slice(self, slice: &mut [f64])
pub fn write_to_slice(self, slice: &mut [f64])
Writes the elements of self to the first N elements in slice.
Panics
Panics if slice is less than N elements long.
Sourcepub fn abs(self) -> DVec4
pub fn abs(self) -> DVec4
Returns a vector containing the absolute value of each element of self.
Sourcepub fn signum(self) -> DVec4
pub fn signum(self) -> DVec4
Returns a vector with elements representing the sign of self.
1.0if the number is positive,+0.0orINFINITY-1.0if the number is negative,-0.0orNEG_INFINITYNANif the number isNAN
Sourcepub fn is_finite(self) -> bool
pub fn is_finite(self) -> bool
Returns true if, and only if, all elements are finite. If any element is either
NaN, positive or negative infinity, this will return false.
Sourcepub fn is_nan_mask(self) -> BVec4
pub fn is_nan_mask(self) -> BVec4
Performs is_nan on each element of self, returning a vector mask of the results.
In other words, this computes [x.is_nan(), y.is_nan(), z.is_nan(), w.is_nan()].
Sourcepub fn length_squared(self) -> f64
pub fn length_squared(self) -> f64
Computes the squared length of self.
This is faster than length() as it avoids a square root operation.
Sourcepub fn length_recip(self) -> f64
pub fn length_recip(self) -> f64
Computes 1.0 / length().
For valid results, self must not be of length zero.
Sourcepub fn distance(self, other: DVec4) -> f64
pub fn distance(self, other: DVec4) -> f64
Computes the Euclidean distance between two points in space.
Sourcepub fn distance_squared(self, other: DVec4) -> f64
pub fn distance_squared(self, other: DVec4) -> f64
Compute the squared euclidean distance between two points in space.
Sourcepub fn normalize(self) -> DVec4
pub fn normalize(self) -> DVec4
Returns self normalized to length 1.0.
For valid results, self must not be of length zero, nor very close to zero.
See also Self::try_normalize and Self::normalize_or_zero.
Panics
Will panic if self is zero length when glam_assert is enabled.
Sourcepub fn try_normalize(self) -> Option<DVec4>
pub fn try_normalize(self) -> Option<DVec4>
Returns self normalized to length 1.0 if possible, else returns None.
In particular, if the input is zero (or very close to zero), or non-finite,
the result of this operation will be None.
See also Self::normalize_or_zero.
Sourcepub fn normalize_or_zero(self) -> DVec4
pub fn normalize_or_zero(self) -> DVec4
Returns self normalized to length 1.0 if possible, else returns zero.
In particular, if the input is zero (or very close to zero), or non-finite, the result of this operation will be zero.
See also Self::try_normalize.
Sourcepub fn is_normalized(self) -> bool
pub fn is_normalized(self) -> bool
Returns whether self is length 1.0 or not.
Uses a precision threshold of 1e-6.
Sourcepub fn project_onto(self, other: DVec4) -> DVec4
pub fn project_onto(self, other: DVec4) -> DVec4
Returns the vector projection of self onto other.
other must be of non-zero length.
Panics
Will panic if other is zero length when glam_assert is enabled.
Sourcepub fn reject_from(self, other: DVec4) -> DVec4
pub fn reject_from(self, other: DVec4) -> DVec4
Returns the vector rejection of self from other.
The vector rejection is the vector perpendicular to the projection of self onto
other, in other words the result of self - self.project_onto(other).
other must be of non-zero length.
Panics
Will panic if other has a length of zero when glam_assert is enabled.
Sourcepub fn project_onto_normalized(self, other: DVec4) -> DVec4
pub fn project_onto_normalized(self, other: DVec4) -> DVec4
Returns the vector projection of self onto other.
other must be normalized.
Panics
Will panic if other is not normalized when glam_assert is enabled.
Sourcepub fn reject_from_normalized(self, other: DVec4) -> DVec4
pub fn reject_from_normalized(self, other: DVec4) -> DVec4
Returns the vector rejection of self from other.
The vector rejection is the vector perpendicular to the projection of self onto
other, in other words the result of self - self.project_onto(other).
other must be normalized.
Panics
Will panic if other is not normalized when glam_assert is enabled.
Sourcepub fn round(self) -> DVec4
pub fn round(self) -> DVec4
Returns a vector containing the nearest integer to a number for each element of self.
Round half-way cases away from 0.0.
Sourcepub fn floor(self) -> DVec4
pub fn floor(self) -> DVec4
Returns a vector containing the largest integer less than or equal to a number for each
element of self.
Sourcepub fn ceil(self) -> DVec4
pub fn ceil(self) -> DVec4
Returns a vector containing the smallest integer greater than or equal to a number for
each element of self.
Sourcepub fn fract(self) -> DVec4
pub fn fract(self) -> DVec4
Returns a vector containing the fractional part of the vector, e.g. self - self.floor().
Note that this is fast but not precise for large numbers.
Sourcepub fn exp(self) -> DVec4
pub fn exp(self) -> DVec4
Returns a vector containing e^self (the exponential function) for each element of
self.
Sourcepub fn powf(self, n: f64) -> DVec4
pub fn powf(self, n: f64) -> DVec4
Returns a vector containing each element of self raised to the power of n.
Sourcepub fn recip(self) -> DVec4
pub fn recip(self) -> DVec4
Returns a vector containing the reciprocal 1.0/n of each element of self.
Sourcepub fn lerp(self, other: DVec4, s: f64) -> DVec4
pub fn lerp(self, other: DVec4, s: f64) -> DVec4
Performs a linear interpolation between self and other based on the value s.
When s is 0.0, the result will be equal to self. When s is 1.0, the result
will be equal to other.
Sourcepub fn abs_diff_eq(self, other: DVec4, max_abs_diff: f64) -> bool
pub fn abs_diff_eq(self, other: DVec4, max_abs_diff: f64) -> bool
Returns true if the absolute difference of all elements between self and other is
less than or equal to max_abs_diff.
This can be used to compare if two vectors contain similar elements. It works best when
comparing with a known value. The max_abs_diff that should be used used depends on
the values being compared against.
For more see comparing floating point numbers.
Sourcepub fn clamp_length(self, min: f64, max: f64) -> DVec4
pub fn clamp_length(self, min: f64, max: f64) -> DVec4
Returns a vector with a length no less than min and no more than max
Panics
Will panic if min is greater than max when glam_assert is enabled.
Sourcepub fn clamp_length_max(self, max: f64) -> DVec4
pub fn clamp_length_max(self, max: f64) -> DVec4
Returns a vector with a length no more than max
Sourcepub fn clamp_length_min(self, min: f64) -> DVec4
pub fn clamp_length_min(self, min: f64) -> DVec4
Returns a vector with a length no less than min